@@ -22,6 +22,12 @@ from os import path
 from .deprecation import RemovedInNextVersionWarning
 from .deprecation import RemovedInSphinx20Warning
 
+if False:
+    # For type annotation
+    # note: Don't use typing.TYPE_CHECK here (for py27 and py34).
+    from typing import Any  # NOQA
+
+
 # by default, all DeprecationWarning under sphinx package will be emit.
 # Users can avoid this by using environment variable: PYTHONWARNINGS=
 if 'PYTHONWARNINGS' not in os.environ:
@@ -31,13 +37,18 @@ if 'PYTHONWARNINGS' not in os.environ:
 warnings.filterwarnings('ignore', "'U' mode is deprecated",
                         DeprecationWarning, module='docutils.io')
 
-__version__ = '1.7.3'
-__released__ = '1.7.3'  # used when Sphinx builds its own docs
+__version__ = '1.8.0'
+__released__ = '1.8.0'  # used when Sphinx builds its own docs
 
-# version info for better programmatic use
-# possible values for 3rd element: 'alpha', 'beta', 'rc', 'final'
-# 'final' has 0 as the last element
-version_info = (1, 7, 3, 'final', 0)
+#: Version info for better programmatic use.
+#:
+#: A tuple of five elements; for Sphinx version 1.2.1 beta 3 this would be
+#: ``(1, 2, 1, 'beta', 3)``. The fourth element can be one of: ``alpha``,
+#: ``beta``, ``rc``, ``final``. ``final`` always has 0 as the last element.
+#:
+#: .. versionadded:: 1.2
+#:    Before version 1.2, check the string ``sphinx.__version__``.
+version_info = (1, 8, 0, 'final', 0)
 
 package_dir = path.abspath(path.dirname(__file__))
 
@@ -50,8 +61,7 @@ if __version__.endswith('+'):
     __version__ = __version__[:-1]  # remove '+' for PEP-440 version spec.
     try:
         import subprocess
-        p = subprocess.Popen(['git', 'show', '-s', '--pretty=format:%h',
-                              path.join(package_dir, '..')],
+        p = subprocess.Popen(['git', 'show', '-s', '--pretty=format:%h'],
                              stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         out, err = p.communicate()
         if out:
@@ -60,15 +70,16 @@ if __version__.endswith('+'):
         pass
 
 
-def main(*args, **kwargs):
+def main(argv=sys.argv):  # type: ignore
+    # type: (List[unicode]) -> int
     from .cmd import build
     warnings.warn(
         '`sphinx.main()` has moved to `sphinx.cmd.build.main()`.',
         RemovedInSphinx20Warning,
         stacklevel=2,
     )
-    args = args[1:]  # skip first argument to adjust arguments (refs: #4615)
-    return build.main(*args, **kwargs)
+    argv = argv[1:]  # skip first argument to adjust arguments (refs: #4615)
+    return build.main(argv)
 
 
 def build_main(argv=sys.argv):
