@@ -15,13 +15,14 @@ import inspect
 import re
 import sys
 import warnings
+from typing import Any
 
 from docutils.statemachine import ViewList
 from six import iteritems, itervalues, text_type, class_types, string_types
 
 import sphinx
-from sphinx.application import ExtensionError
 from sphinx.deprecation import RemovedInSphinx20Warning
+from sphinx.errors import ExtensionError
 from sphinx.ext.autodoc.importer import mock, import_object, get_object_members
 from sphinx.ext.autodoc.importer import _MockImporter  # to keep compatibility  # NOQA
 from sphinx.ext.autodoc.inspector import format_annotation, formatargspec  # to keep compatibility  # NOQA
@@ -32,7 +33,7 @@ from sphinx.util import rpartition, force_decode
 from sphinx.util.docstrings import prepare_docstring
 from sphinx.util.inspect import Signature, isdescriptor, safe_getmembers, \
     safe_getattr, object_description, is_builtin_class_method, \
-    isenumattribute, isclassmethod, isstaticmethod, getdoc
+    isenumattribute, isclassmethod, isstaticmethod, isfunction, isbuiltin, ispartial, getdoc
 
 if False:
     # For type annotation
@@ -41,6 +42,7 @@ if False:
     from docutils import nodes  # NOQA
     from docutils.utils import Reporter  # NOQA
     from sphinx.application import Sphinx  # NOQA
+    from sphinx.config import Config  # NOQA
     from sphinx.environment import BuildEnvironment  # NOQA
     from sphinx.ext.autodoc.directive import DocumenterBridge  # NOQA
 
@@ -108,6 +110,20 @@ def bool_option(arg):
     return True
 
 
+def merge_special_members_option(options):
+    # type: (Dict) -> None
+    """Merge :special-members: option to :members: option."""
+    if 'special-members' in options and options['special-members'] is not ALL:
+        if options.get('members') is ALL:
+            pass
+        elif options.get('members'):
+            for member in options['special-members']:
+                if member not in options['members']:
+                    options['members'].append(member)
+        else:
+            options['members'] = options['special-members']
+
+
 class AutodocReporter(object):
     """
     A reporter replacement that assigns the correct source name
@@ -340,7 +356,7 @@ class Documenter(object):
             explicit_modname, path, base, args, retann = \
                 py_ext_sig_re.match(self.name).groups()  # type: ignore
         except AttributeError:
-            logger.warning('invalid signature for auto%s (%r)' % (self.objtype, self.name))
+            logger.warning(__('invalid signature for auto%s (%r)') % (self.objtype, self.name))
             return False
 
         # support explicit module and class name separation via ::
@@ -399,7 +415,9 @@ class Documenter(object):
             return True
 
         modname = self.get_attr(self.object, '__module__', None)
-        if modname and modname != self.modname:
+        if ispartial(self.object) and modname == '_functools':  # for pypy
+            return True
+        elif modname and modname != self.modname:
             return False
         return True
 
@@ -437,7 +455,7 @@ class Documenter(object):
             try:
                 args = self.format_args()
             except Exception as err:
-                logger.warning('error while formatting arguments for %s: %s' %
+                logger.warning(__('error while formatting arguments for %s: %s') %
                                (self.fullname, err))
                 args = None
 
@@ -473,9 +491,8 @@ class Documenter(object):
     def get_doc(self, encoding=None, ignore=1):
         # type: (unicode, int) -> List[List[unicode]]
         """Decode and return lines of the docstring(s) for the object."""
-        docstring = self.get_attr(self.object, '__doc__', None)
-        if docstring is None and self.env.config.autodoc_inherit_docstrings:
-            docstring = getdoc(self.object)
+        docstring = getdoc(self.object, self.get_attr,
+                           self.env.config.autodoc_inherit_docstrings)
         # make sure we have Unicode docstrings, then sanitize and split
         # into lines
         if isinstance(docstring, text_type):
@@ -560,7 +577,7 @@ class Documenter(object):
                 if name in members:
                     selected.append((name, members[name].value))
                 else:
-                    logger.warning('missing attribute %s in object %s' %
+                    logger.warning(__('missing attribute %s in object %s') %
                                    (name, self.fullname))
             return False, sorted(selected)
         elif self.options.inherited_members:
@@ -599,9 +616,7 @@ class Documenter(object):
             # if isattr is True, the member is documented as an attribute
             isattr = False
 
-            doc = self.get_attr(member, '__doc__', None)
-            if doc is None and self.env.config.autodoc_inherit_docstrings:
-                doc = getdoc(member)
+            doc = getdoc(member, self.get_attr, self.env.config.autodoc_inherit_docstrings)
 
             # if the member __doc__ is the same as self's __doc__, it's just
             # inherited and therefore not the member's doc
@@ -679,8 +694,13 @@ class Documenter(object):
 
         # remove members given by exclude-members
         if self.options.exclude_members:
-            members = [(membername, member) for (membername, member) in members
-                       if membername not in self.options.exclude_members]
+            members = [
+                (membername, member) for (membername, member) in members
+                if (
+                    self.options.exclude_members is ALL or
+                    membername not in self.options.exclude_members
+                )
+            ]
 
         # document non-skipped members
         memberdocumenters = []  # type: List[Tuple[Documenter, bool]]
@@ -737,9 +757,9 @@ class Documenter(object):
         if not self.parse_name():
             # need a module to import
             logger.warning(
-                'don\'t know which module to import for autodocumenting '
-                '%r (try placing a "module" or "currentmodule" directive '
-                'in the document, or giving an explicit module name)' %
+                __('don\'t know which module to import for autodocumenting '
+                   '%r (try placing a "module" or "currentmodule" directive '
+                   'in the document, or giving an explicit module name)') %
                 self.name)
             return
 
@@ -817,6 +837,11 @@ class ModuleDocumenter(Documenter):
         'imported-members': bool_option, 'ignore-module-all': bool_option
     }  # type: Dict[unicode, Callable]
 
+    def __init__(self, *args):
+        # type: (Any) -> None
+        super(ModuleDocumenter, self).__init__(*args)
+        merge_special_members_option(self.options)
+
     @classmethod
     def can_document_member(cls, member, membername, isattr, parent):
         # type: (Any, unicode, bool, Any) -> bool
@@ -826,15 +851,15 @@ class ModuleDocumenter(Documenter):
     def resolve_name(self, modname, parents, path, base):
         # type: (str, Any, str, Any) -> Tuple[str, List[unicode]]
         if modname is not None:
-            logger.warning('"::" in automodule name doesn\'t make sense')
+            logger.warning(__('"::" in automodule name doesn\'t make sense'))
         return (path or '') + base, []
 
     def parse_name(self):
         # type: () -> bool
         ret = Documenter.parse_name(self)
         if self.args or self.retann:
-            logger.warning('signature arguments or return annotation '
-                           'given for automodule %s' % self.fullname)
+            logger.warning(__('signature arguments or return annotation '
+                              'given for automodule %s') % self.fullname)
         return ret
 
     def add_directive_header(self, sig):
@@ -867,8 +892,8 @@ class ModuleDocumenter(Documenter):
                 if not isinstance(memberlist, (list, tuple)) or not \
                    all(isinstance(entry, string_types) for entry in memberlist):
                     logger.warning(
-                        '__all__ should be a list of strings, not %r '
-                        '(in module %s) -- ignoring __all__' %
+                        __('__all__ should be a list of strings, not %r '
+                           '(in module %s) -- ignoring __all__') %
                         (memberlist, self.fullname))
                     # fall back to all members
                     return True, safe_getmembers(self.object)
@@ -880,8 +905,8 @@ class ModuleDocumenter(Documenter):
                 ret.append((mname, safe_getattr(self.object, mname)))
             except AttributeError:
                 logger.warning(
-                    'missing attribute mentioned in :members: or __all__: '
-                    'module %s, attribute %s' %
+                    __('missing attribute mentioned in :members: or __all__: '
+                       'module %s, attribute %s') %
                     (safe_getattr(self.object, '__name__', '???'), mname))
         return False, ret
 
@@ -1022,12 +1047,11 @@ class FunctionDocumenter(DocstringSignatureMixin, ModuleLevelDocumenter):  # typ
     @classmethod
     def can_document_member(cls, member, membername, isattr, parent):
         # type: (Any, unicode, bool, Any) -> bool
-        return inspect.isfunction(member) or inspect.isbuiltin(member)
+        return isfunction(member) or isbuiltin(member)
 
     def format_args(self):
         # type: () -> unicode
-        if inspect.isbuiltin(self.object) or \
-                inspect.ismethoddescriptor(self.object):
+        if isbuiltin(self.object) or inspect.ismethoddescriptor(self.object):
             # cannot introspect arguments of a C function or method
             return None
         try:
@@ -1070,6 +1094,11 @@ class ClassDocumenter(DocstringSignatureMixin, ModuleLevelDocumenter):  # type:
         'private-members': bool_option, 'special-members': members_option,
     }  # type: Dict[unicode, Callable]
 
+    def __init__(self, *args):
+        # type: (Any) -> None
+        super(ClassDocumenter, self).__init__(*args)
+        merge_special_members_option(self.options)
+
     @classmethod
     def can_document_member(cls, member, membername, isattr, parent):
         # type: (Any, unicode, bool, Any) -> bool
@@ -1095,7 +1124,7 @@ class ClassDocumenter(DocstringSignatureMixin, ModuleLevelDocumenter):  # type:
         # __init__ written in C?
         if initmeth is None or \
                 is_builtin_class_method(self.object, '__init__') or \
-                not(inspect.ismethod(initmeth) or inspect.isfunction(initmeth)):
+                not(inspect.ismethod(initmeth) or isfunction(initmeth)):
             return None
         try:
             return Signature(initmeth, bound_method=True, has_retval=False).format_args()
@@ -1201,6 +1230,7 @@ class ClassDocumenter(DocstringSignatureMixin, ModuleLevelDocumenter):  # type:
 
     def generate(self, more_content=None, real_modname=None,
                  check_module=False, all_members=False):
+        # type: (Any, str, bool, bool) -> None
         # Do not pass real_modname and use the name from the __module__
         # attribute of the class.
         # If a class gets imported into the module real_modname
@@ -1264,6 +1294,11 @@ class DataDocumenter(ModuleLevelDocumenter):
         # type: (bool) -> None
         pass
 
+    def get_real_modname(self):
+        # type: () -> str
+        return self.get_attr(self.parent or self.object, '__module__', None) \
+            or self.modname
+
 
 class MethodDocumenter(DocstringSignatureMixin, ClassLevelDocumenter):  # type: ignore
     """
@@ -1304,8 +1339,7 @@ class MethodDocumenter(DocstringSignatureMixin, ClassLevelDocumenter):  # type:
 
     def format_args(self):
         # type: () -> unicode
-        if inspect.isbuiltin(self.object) or \
-                inspect.ismethoddescriptor(self.object):
+        if isbuiltin(self.object) or inspect.ismethoddescriptor(self.object):
             # can never get arguments of a C function or method
             return None
         if isstaticmethod(self.object, cls=self.parent, name=self.object_name):
@@ -1336,7 +1370,8 @@ class AttributeDocumenter(DocstringStripSignatureMixin, ClassLevelDocumenter):
 
     @staticmethod
     def is_function_or_method(obj):
-        return inspect.isfunction(obj) or inspect.isbuiltin(obj) or inspect.ismethod(obj)
+        # type: (Any) -> bool
+        return isfunction(obj) or isbuiltin(obj) or inspect.ismethod(obj)
 
     @classmethod
     def can_document_member(cls, member, membername, isattr, parent):
@@ -1436,18 +1471,22 @@ class InstanceAttributeDocumenter(AttributeDocumenter):
 
 class DeprecatedDict(dict):
     def __init__(self, message):
+        # type: (str) -> None
         self.message = message
         super(DeprecatedDict, self).__init__()
 
     def __setitem__(self, key, value):
+        # type: (unicode, Any) -> None
         warnings.warn(self.message, RemovedInSphinx20Warning)
         super(DeprecatedDict, self).__setitem__(key, value)
 
     def setdefault(self, key, default=None):
+        # type: (unicode, Any) -> None
         warnings.warn(self.message, RemovedInSphinx20Warning)
         super(DeprecatedDict, self).setdefault(key, default)
 
-    def update(self, other=None):
+    def update(self, other=None):  # type: ignore
+        # type: (Dict) -> None
         warnings.warn(self.message, RemovedInSphinx20Warning)
         super(DeprecatedDict, self).update(other)
 
@@ -1517,6 +1556,25 @@ def autodoc_attrgetter(app, obj, name, *defargs):
     return safe_getattr(obj, name, *defargs)
 
 
+def merge_autodoc_default_flags(app, config):
+    # type: (Sphinx, Config) -> None
+    """This merges the autodoc_default_flags to autodoc_default_options."""
+    if not config.autodoc_default_flags:
+        return
+
+    logger.warning(__('autodoc_default_flags is now deprecated. '
+                      'Please use autodoc_default_options instead.'))
+
+    for option in config.autodoc_default_flags:
+        if isinstance(option, string_types):
+            config.autodoc_default_options[option] = None
+        else:
+            logger.warning(
+                __("Ignoring invalid option in autodoc_default_flags: %r"),
+                option
+            )
+
+
 def setup(app):
     # type: (Sphinx) -> Dict[unicode, Any]
     app.add_autodocumenter(ModuleDocumenter)
@@ -1531,6 +1589,7 @@ def setup(app):
     app.add_config_value('autoclass_content', 'class', True)
     app.add_config_value('autodoc_member_order', 'alphabetic', True)
     app.add_config_value('autodoc_default_flags', [], True)
+    app.add_config_value('autodoc_default_options', {}, True)
     app.add_config_value('autodoc_docstring_signature', True, True)
     app.add_config_value('autodoc_mock_imports', [], True)
     app.add_config_value('autodoc_warningiserror', True, True)
@@ -1539,16 +1598,6 @@ def setup(app):
     app.add_event('autodoc-process-signature')
     app.add_event('autodoc-skip-member')
 
+    app.connect('config-inited', merge_autodoc_default_flags)
+
     return {'version': sphinx.__display_version__, 'parallel_read_safe': True}
-
-
-class testcls:
-    """test doc string"""
-
-    def __getattr__(self, x):
-        # type: (Any) -> Any
-        return x
-
-    def __setattr__(self, x, y):
-        # type: (Any, Any) -> None
-        """Attr setter."""
