@@ -5,7 +5,7 @@
 
     Create a full-text search index for offline search.
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 import re
@@ -65,7 +65,7 @@ var Stemmer = function() {
 }
 """                             # type: unicode
 
-    _word_re = re.compile(r'\w+(?u)')
+    _word_re = re.compile(r'(?u)\w+')
 
     def __init__(self, options):
         # type: (Dict) -> None
@@ -331,13 +331,13 @@ class IndexBuilder(object):
         for domainname, domain in sorted(iteritems(self.env.domains)):
             for fullname, dispname, type, docname, anchor, prio in \
                     sorted(domain.get_objects()):
-                # XXX use dispname?
                 if docname not in fn2index:
                     continue
                 if prio < 0:
                     continue
                 fullname = htmlescape(fullname)
-                prefix, name = rpartition(fullname, '.')
+                dispname = htmlescape(dispname)
+                prefix, name = rpartition(dispname, '.')
                 pdict = rv.setdefault(prefix, {})
                 try:
                     typeindex = otypes[domainname, type]
