@@ -15,16 +15,21 @@ import re
 import sys
 import typing
 from collections import OrderedDict
+from functools import partial
 
 from six import PY2, PY3, StringIO, binary_type, string_types, itervalues
 from six.moves import builtins
 
 from sphinx.util import force_decode
+from sphinx.util import logging
+from sphinx.util.pycompat import NoneType
 
 if False:
     # For type annotation
     from typing import Any, Callable, Dict, List, Tuple, Type  # NOQA
 
+logger = logging.getLogger(__name__)
+
 memory_address_re = re.compile(r' at 0x[0-9a-f]{8,16}(?=>)', re.IGNORECASE)
 
 
@@ -98,8 +103,6 @@ if PY3:
                                    kwonlyargs, kwdefaults, annotations)
 
 else:  # 2.7
-    from functools import partial
-
     def getargspec(func):
         # type: (Any) -> Any
         """Like inspect.getargspec but supports functools.partial as well."""
@@ -154,6 +157,12 @@ def isenumattribute(x):
     return isinstance(x, enum.Enum)
 
 
+def ispartial(obj):
+    # type: (Any) -> bool
+    """Check if the object is partial."""
+    return isinstance(obj, partial)
+
+
 def isclassmethod(obj):
     # type: (Any) -> bool
     """Check if the object is classmethod."""
@@ -176,8 +185,8 @@ def isstaticmethod(obj, cls=None, name=None):
     elif cls and name:
         # trace __mro__ if the method is defined in parent class
         #
-        # .. note:: This only works with new style classes.
-        for basecls in getattr(cls, '__mro__', []):
+        # .. note:: This only works well with new style classes.
+        for basecls in getattr(cls, '__mro__', [cls]):
             meth = basecls.__dict__.get(name)
             if meth:
                 if isinstance(meth, staticmethod):
@@ -197,6 +206,18 @@ def isdescriptor(x):
     return False
 
 
+def isfunction(obj):
+    # type: (Any) -> bool
+    """Check if the object is function."""
+    return inspect.isfunction(obj) or ispartial(obj) and inspect.isfunction(obj.func)
+
+
+def isbuiltin(obj):
+    # type: (Any) -> bool
+    """Check if the object is builtin."""
+    return inspect.isbuiltin(obj) or ispartial(obj) and inspect.isbuiltin(obj.func)
+
+
 def safe_getattr(obj, name, *defargs):
     # type: (Any, unicode, unicode) -> object
     """A getattr() that turns all exceptions into AttributeErrors."""
@@ -241,11 +262,22 @@ def object_description(object):
     if isinstance(object, dict):
         try:
             sorted_keys = sorted(object)
-        except TypeError:
+        except Exception:
             pass  # Cannot sort dict keys, fall back to generic repr
         else:
-            items = ("%r: %r" % (key, object[key]) for key in sorted_keys)
+            items = ("%s: %s" %
+                     (object_description(key), object_description(object[key]))
+                     for key in sorted_keys)
             return "{%s}" % ", ".join(items)
+    if isinstance(object, set):
+        try:
+            sorted_values = sorted(object)
+        except TypeError:
+            pass  # Cannot sort set values, fall back to generic repr
+        else:
+            template = "{%s}" if PY3 else "set([%s])"
+            return template % ", ".join(object_description(x)
+                                        for x in sorted_values)
     try:
         s = repr(object)
     except Exception:
@@ -298,8 +330,8 @@ class Signature(object):
     its return annotation.
     """
 
-    def __init__(self, subject, bound_method=False):
-        # type: (Callable, bool) -> None
+    def __init__(self, subject, bound_method=False, has_retval=True):
+        # type: (Callable, bool, bool) -> None
         # check subject is not a built-in class (ex. int, str)
         if (isinstance(subject, type) and
                 is_builtin_class_method(subject, "__new__") and
@@ -307,13 +339,17 @@ class Signature(object):
             raise TypeError("can't compute signature for built-in type {}".format(subject))
 
         self.subject = subject
+        self.has_retval = has_retval
         self.partialmethod_with_noargs = False
 
         if PY3:
             try:
                 self.signature = inspect.signature(subject)
             except IndexError:
-                if hasattr(subject, '_partialmethod'):  # partialmethod with no argument
+                # Until python 3.6.4, cpython has been crashed on inspection for
+                # partialmethods not having any arguments.
+                # https://bugs.python.org/issue33009
+                if hasattr(subject, '_partialmethod'):
                     self.signature = None
                     self.partialmethod_with_noargs = True
                 else:
@@ -323,8 +359,14 @@ class Signature(object):
 
         try:
             self.annotations = typing.get_type_hints(subject)  # type: ignore
-        except Exception:
-            self.annotations = {}
+        except Exception as exc:
+            if (3, 5, 0) <= sys.version_info < (3, 5, 3) and isinstance(exc, AttributeError):
+                # python 3.5.2 raises ValueError for partial objects.
+                self.annotations = {}
+            else:
+                logger.warning('Invalid type annotation found on %r. Ingored: %r',
+                               subject, exc)
+                self.annotations = {}
 
         if bound_method:
             # client gives a hint that the subject is a bound method
@@ -372,7 +414,10 @@ class Signature(object):
     def return_annotation(self):
         # type: () -> Any
         if PY3 and self.signature:
-            return self.signature.return_annotation
+            if self.has_retval:
+                return self.signature.return_annotation
+            else:
+                return inspect.Parameter.empty
         else:
             return None
 
@@ -426,8 +471,7 @@ class Signature(object):
         if PY2 or self.return_annotation is inspect.Parameter.empty:
             return '(%s)' % ', '.join(args)
         else:
-            if isinstance(self.return_annotation, string_types) and \
-                    'return' in self.annotations:
+            if 'return' in self.annotations:
                 annotation = self.format_annotation(self.annotations['return'])
             else:
                 annotation = self.format_annotation(self.return_annotation)
@@ -443,6 +487,19 @@ class Signature(object):
 
         Displaying complex types from ``typing`` relies on its private API.
         """
+        if isinstance(annotation, string_types):
+            return annotation  # type: ignore
+        elif isinstance(annotation, typing.TypeVar):  # type: ignore
+            return annotation.__name__
+        elif not annotation:
+            return repr(annotation)
+        elif annotation is NoneType:  # type: ignore
+            return 'None'
+        elif getattr(annotation, '__module__', None) == 'builtins':
+            return annotation.__qualname__
+        elif annotation is Ellipsis:
+            return '...'
+
         if sys.version_info >= (3, 7):  # py37+
             return self.format_annotation_new(annotation)
         else:
@@ -452,22 +509,13 @@ class Signature(object):
         # type: (Any) -> str
         """format_annotation() for py37+"""
         module = getattr(annotation, '__module__', None)
-        if isinstance(annotation, string_types):
-            return annotation  # type: ignore
-        elif isinstance(annotation, typing.TypeVar):  # type: ignore
-            return annotation.__name__
-        elif not annotation:
-            return repr(annotation)
-        elif module == 'builtins':
-            return annotation.__qualname__
-        elif annotation is Ellipsis:
-            return '...'
-
         if module == 'typing':
             if getattr(annotation, '_name', None):
                 qualname = annotation._name
             elif getattr(annotation, '__qualname__', None):
                 qualname = annotation.__qualname__
+            elif getattr(annotation, '__forward_arg__', None):
+                qualname = annotation.__forward_arg__
             else:
                 qualname = self.format_annotation(annotation.__origin__)  # ex. Union
         elif hasattr(annotation, '__qualname__'):
@@ -477,8 +525,11 @@ class Signature(object):
 
         if getattr(annotation, '__args__', None):
             if qualname == 'Union':
-                args = ', '.join(self.format_annotation(a) for a in annotation.__args__)
-                return '%s[%s]' % (qualname, args)
+                if len(annotation.__args__) == 2 and annotation.__args__[1] is NoneType:  # type: ignore  # NOQA
+                    return 'Optional[%s]' % self.format_annotation(annotation.__args__[0])
+                else:
+                    args = ', '.join(self.format_annotation(a) for a in annotation.__args__)
+                    return '%s[%s]' % (qualname, args)
             elif qualname == 'Callable':
                 args = ', '.join(self.format_annotation(a) for a in annotation.__args__[:-1])
                 returns = self.format_annotation(annotation.__args__[-1])
@@ -492,28 +543,30 @@ class Signature(object):
     def format_annotation_old(self, annotation):
         # type: (Any) -> str
         """format_annotation() for py36 or below"""
-        if isinstance(annotation, string_types):
-            return annotation  # type: ignore
-        if isinstance(annotation, typing.TypeVar):  # type: ignore
-            return annotation.__name__
-        if annotation == Ellipsis:
-            return '...'
-        if not isinstance(annotation, type):
-            qualified_name = repr(annotation)
-            if qualified_name.startswith('typing.'):  # for typing.Union
-                return qualified_name.split('.', 1)[1]
+        module = getattr(annotation, '__module__', None)
+        if module == 'typing':
+            if getattr(annotation, '_name', None):
+                qualname = annotation._name
+            elif getattr(annotation, '__qualname__', None):
+                qualname = annotation.__qualname__
+            elif getattr(annotation, '__forward_arg__', None):
+                qualname = annotation.__forward_arg__
+            elif getattr(annotation, '__origin__', None):
+                qualname = self.format_annotation(annotation.__origin__)  # ex. Union
             else:
-                return qualified_name
-
-        if not annotation:
-            qualified_name = repr(annotation)
-        elif annotation.__module__ == 'typing':
-            qualified_name = annotation.__qualname__  # type: ignore
+                qualname = repr(annotation).replace('typing.', '')
+        elif hasattr(annotation, '__qualname__'):
+            qualname = '%s.%s' % (module, annotation.__qualname__)
         else:
-            qualified_name = (annotation.__module__ + '.' + annotation.__qualname__)  # type: ignore  # NOQA
+            qualname = repr(annotation)
 
-        if annotation.__module__ == 'builtins':
-            return annotation.__qualname__  # type: ignore
+        if (hasattr(typing, 'TupleMeta') and
+                isinstance(annotation, typing.TupleMeta) and  # type: ignore
+                not hasattr(annotation, '__tuple_params__')):
+            # This is for Python 3.6+, 3.5 case is handled below
+            params = annotation.__args__
+            param_str = ', '.join(self.format_annotation(p) for p in params)
+            return '%s[%s]' % (qualname, param_str)
         elif (hasattr(typing, 'GenericMeta') and  # for py36 or below
               isinstance(annotation, typing.GenericMeta)):
             # In Python 3.5.2+, all arguments are stored in __args__,
@@ -529,19 +582,32 @@ class Signature(object):
                     args = ', '.join(self.format_annotation(arg) for arg
                                      in annotation.__args__[:-1])  # type: ignore
                     result = self.format_annotation(annotation.__args__[-1])  # type: ignore
-                    return '%s[[%s], %s]' % (qualified_name, args, result)
+                    return '%s[[%s], %s]' % (qualname, args, result)
             elif hasattr(annotation, '__parameters__'):
                 params = annotation.__parameters__  # type: ignore
             if params is not None:
                 param_str = ', '.join(self.format_annotation(p) for p in params)
-                return '%s[%s]' % (qualified_name, param_str)
+                return '%s[%s]' % (qualname, param_str)
         elif (hasattr(typing, 'UnionMeta') and  # for py35 or below
               isinstance(annotation, typing.UnionMeta) and  # type: ignore
               hasattr(annotation, '__union_params__')):
             params = annotation.__union_params__
             if params is not None:
-                param_str = ', '.join(self.format_annotation(p) for p in params)
-                return '%s[%s]' % (qualified_name, param_str)
+                if len(params) == 2 and params[1] is NoneType:  # type: ignore
+                    return 'Optional[%s]' % self.format_annotation(params[0])
+                else:
+                    param_str = ', '.join(self.format_annotation(p) for p in params)
+                    return '%s[%s]' % (qualname, param_str)
+        elif (hasattr(typing, 'Union') and  # for py36
+              hasattr(annotation, '__origin__') and
+              annotation.__origin__ is typing.Union):
+            params = annotation.__args__
+            if params is not None:
+                if len(params) == 2 and params[1] is NoneType:  # type: ignore
+                    return 'Optional[%s]' % self.format_annotation(params[0])
+                else:
+                    param_str = ', '.join(self.format_annotation(p) for p in params)
+                    return 'Union[%s]' % param_str
         elif (hasattr(typing, 'CallableMeta') and  # for py36 or below
               isinstance(annotation, typing.CallableMeta) and  # type: ignore
               getattr(annotation, '__args__', None) is not None and
@@ -549,13 +615,13 @@ class Signature(object):
             # Skipped in the case of plain typing.Callable
             args = annotation.__args__
             if args is None:
-                return qualified_name
+                return qualname
             elif args is Ellipsis:
                 args_str = '...'
             else:
                 formatted_args = (self.format_annotation(a) for a in args)
                 args_str = '[%s]' % ', '.join(formatted_args)
-            return '%s[%s, %s]' % (qualified_name,
+            return '%s[%s, %s]' % (qualname,
                                    args_str,
                                    self.format_annotation(annotation.__result__))
         elif (hasattr(typing, 'TupleMeta') and  # for py36 or below
@@ -567,19 +633,20 @@ class Signature(object):
                 param_strings = [self.format_annotation(p) for p in params]
                 if annotation.__tuple_use_ellipsis__:
                     param_strings.append('...')
-                return '%s[%s]' % (qualified_name,
+                return '%s[%s]' % (qualname,
                                    ', '.join(param_strings))
 
-        return qualified_name
+        return qualname
 
 
 if sys.version_info >= (3, 5):
-    getdoc = inspect.getdoc
+    _getdoc = inspect.getdoc
 else:
     # code copied from the inspect.py module of the standard library
     # of Python 3.5
 
     def _findclass(func):
+        # type: (Any) -> Any
         cls = sys.modules.get(func.__module__)
         if cls is None:
             return None
@@ -593,6 +660,7 @@ else:
         return cls
 
     def _finddoc(obj):
+        # type: (Any) -> unicode
         if inspect.isclass(obj):
             for base in obj.__mro__:
                 if base is not object:
@@ -652,7 +720,8 @@ else:
                 return doc
         return None
 
-    def getdoc(object):
+    def _getdoc(object):
+        # type: (Any) -> unicode
         """Get the documentation string for an object.
 
         All tabs are expanded to spaces.  To clean up docstrings that are
@@ -670,3 +739,21 @@ else:
         if not isinstance(doc, str):
             return None
         return inspect.cleandoc(doc)
+
+
+def getdoc(obj, attrgetter=safe_getattr, allow_inherited=False):
+    # type: (Any, Callable, bool) -> unicode
+    """Get the docstring for the object.
+
+    This tries to obtain the docstring for some kind of objects additionally:
+
+    * partial functions
+    * inherited docstring
+    """
+    doc = attrgetter(obj, '__doc__', None)
+    if ispartial(obj) and doc == obj.__class__.__doc__:
+        return getdoc(obj.func)
+    elif doc is None and allow_inherited:
+        doc = _getdoc(obj)
+
+    return doc
